@@ -1,8 +1,8 @@
 # Vend::Data - Interchange databases
 #
-# $Id: Data.pm,v 2.36 2004/04/02 17:37:46 mheins Exp $
+# $Id: Data.pm,v 2.54 2005/10/23 14:23:45 mheins Exp $
 # 
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2004 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -62,6 +62,7 @@ update_data
 @EXPORT_OK = qw(update_productbase column_index);
 
 use strict;
+no warnings qw(uninitialized numeric);
 use File::Basename;
 use Vend::Util;
 use Vend::Interpolate;
@@ -73,6 +74,7 @@ BEGIN {
 # SQL
 	if($Global::DBI) {
 		require Vend::Table::DBI;
+		require Vend::Table::DBI_CompositeKey;
 	}
 # END SQL
 # LDAP
@@ -131,6 +133,7 @@ sub instant_database {
 }
 
 sub database_exists_ref {
+	return unless $_[0];
 	return $_[0]->ref() if ref $_[0];
 	return $Vend::Interpolate::Db{$_[0]}
 			if $Vend::Interpolate::Db{$_[0]};
@@ -522,8 +525,9 @@ sub read_salestax {
     my($code, $percent);
 
 	return unless $Vend::Cfg->{SalesTax};
+	return if $Vend::Cfg->{SalesTax} eq 'multi';
 	my $file = $Vend::Cfg->{Special}{'salestax.asc'};
-	$file = Vend::Util::catfile($Vend::Cfg->{ProductDir}, "salestax.asc")
+	$file = Vend::File::catfile($Vend::Cfg->{ProductDir}, "salestax.asc")
 		unless $file;
 
 	$Vend::Cfg->{SalesTaxTable} = {};
@@ -550,6 +554,7 @@ my %Delimiter = (
 	6 => ["\t", "\n"],
 	7 => ["\t", "\n"],
 	8 => ["\t", "\n"],
+	10 => ["\t", "\n"],
 	LINE => ["\n", "\n\n"],
 	'%%%' => ["\n%%\n", "\n%%%\n"],
 	'%%' => ["\n%%\n", "\n%%%\n"],
@@ -615,6 +620,13 @@ use vars '%db_config';
 
 %db_config = (
 # SQL
+		'DBI_CompositeKey' => {
+				qw/
+					Extension			 sql
+					RestrictedImport	 1
+					Class                Vend::Table::DBI_CompositeKey
+				/
+				},
 		'DBI' => {
 				qw/
 					Extension			 sql
@@ -765,13 +777,20 @@ sub create_empty_txt {
 	my ($obj, $database_txt, $delimiter, $record_delim) = @_;
 	return if -f $database_txt;
 	return unless $obj->{CREATE_EMPTY_TXT};
-	if(! ref($obj->{NAME}) eq 'ARRAY') {
-		logError("Cannot create text file with no database NAME parameter");
+	my $ary;
+	if($obj->{CREATE_EMPTY_TXT} =~ /[\s,]\w/) {
+		$ary = [ grep /\S/, split /[\s,]+/, $obj->{CREATE_EMPTY_TXT} ];
+	}
+	else {
+		$ary = $obj->{NAME};
+	}
+	unless (ref($ary) eq 'ARRAY') {
+		logError("Cannot create text file with no database NAME parameter and no field names in CREATE_EMPTY_TXT");
 	}
 	else {
 		$delimiter ||= "\t";
 		$record_delim ||= "\n";
-		my $line = join $obj->{DELIMITER}, @{$obj->{NAME}};
+		my $line = join $obj->{DELIMITER}, @$ary;
 		$line .= $record_delim;
 		Vend::Util::writefile($database_txt, $line);
 	}
@@ -811,7 +830,7 @@ sub import_database {
 	my $class_config;
 	my $db;
 
-	my $no_import = defined $Vend::Cfg->{NoImport}->{$name};
+	my $no_import = defined $Vend::Cfg->{NoImport}->{$name} || $obj->{NO_IMPORT};
 
 	if (defined $Vend::ForceImport{$name}) {
 		undef $no_import;
@@ -1384,7 +1403,20 @@ sub export_database {
 		unlink $f if $f;
 	}
 	else {
-		$db->touch() unless defined $notouch;
+		$db->touch();
+	}
+	if (my $subs = $db->config('POSTEXPORT')) {
+		# Make a copy of the options once to hand off to each sub.
+		my $options = { %$opt, delim => $delim, record_delim => $record_delim };
+		for my $name (@$subs) {
+			my $sub = $Vend::Cfg->{Sub}{$name} || $Global::GlobalSub->{$name}
+				or do {
+					logError("Unknown POSTEXPORT sub '%s' on database '%s'.", $name, $db->name);
+					next;
+				};
+			$sub->($db->name, $file, $options)
+				or logError("Failed call to POSTEXPORT sub '%s' on database '%s'!", $name, $db->name);
+		}
 	}
 	1;
 }
@@ -1537,7 +1569,8 @@ CHAIN:
 					redo CHAIN if ref $row ne 'HASH';
 
 					my $keep;
-					$keep = $row->{$field} if $row->{$field} != 0;
+					$keep = $row->{$field}
+						if length($row->{$field}) && $row->{$field} != 0;
 					for (@breaks) {
 						next unless exists $row->{$_};
 						$test = $_;
@@ -1669,12 +1702,33 @@ sub item_price {
 		)
 	{
 		foreach my $i (@{$Vend::Cfg->{AutoModifier}}) {
-			my ($table,$key) = split /:/, $i;
+			my $attr;
+			my ($table,$key,$foreign) = split /:+/, $i, 3;
+
+			if($table =~ /=/) {
+				($attr, $table) = split /\s*=\s*/, $table, 2;
+			}
+
+			if(! $key and ! $foreign) {
+				$attr ||= $table;
+				$item->{$attr} = item_common($item, $table);
+				next;
+			}
+
 			unless ($key) {
 				$key = $table;
 				$table = $item->{mv_ib};
 			}
-			$item->{$key} = ::tag_data($table, $key, $item->{code});
+
+			$attr ||= $key;
+			$table ||= $Vend::Cfg->{ProductFiles}[0];
+
+			my $select = $foreign ? $item->{$foreign} : $item->{code};
+			$select ||= $item->{code};
+
+#::logDebug("attr=$attr table=$table key=$key select=$select foreign=$foreign");
+			$item->{$attr} = ::tag_data($table, $key, $select);
+#::logDebug("item->$attr=$item->{$attr}");
 		}
 #::logDebug("item=" . ::uneval($item));
 	}
@@ -1701,16 +1755,17 @@ sub item_price {
 
 	my $final = 0;
 	do {
-		my $base;
-		if(not $base = $item->{mv_ib}) {
-			$base = product_code_exists_tag($item->{code})
-				or ($Vend::Cfg->{OnFly} && 'mv_fly')
-				or return undef;
-		}
-
 		my $price;
-		$price = database_field($base, $item->{code}, $Vend::Cfg->{PriceField})
-			if $Vend::Cfg->{PriceField};
+
+		if ($Vend::Cfg->{PriceField}) {
+			my $base;
+			if (not $base = $item->{mv_ib}) {
+				$base = product_code_exists_tag($item->{code})
+					or ($Vend::Cfg->{OnFly} && 'mv_fly')
+					or return undef;
+			}
+			$price = database_field($base, $item->{code}, $Vend::Cfg->{PriceField});
+		}
 
 #::logDebug("price for item before chain $item->{code}=$price PriceField=$Vend::Cfg->{PriceField}");
 		$price = chain_cost($item,$price || $Vend::Cfg->{CommonAdjust});
@@ -1778,7 +1833,7 @@ sub item_field {
 }
 
 sub item_subtotal {
-	item_price($_[0]) * $_[0]->{quantity};
+	item_price($_[0]) * ($_[0]->{quantity} || 0);
 }
 
 sub set_db {
@@ -1833,6 +1888,8 @@ sub update_data {
 	my $prikey		= $CGI::values{mv_data_key};
 	my $decode		= is_yes($CGI::values{mv_data_decode});
 
+	my %skip_for_now;
+
 	my $en_col;
 #::logDebug("data_enable=$::Scratch->{mv_data_enable}, checking");
 	if($::Scratch->{mv_data_enable} =~ /^(\w+):(.*?):/s) {
@@ -1873,6 +1930,7 @@ sub update_data {
 
     my @file_fields = split /[\s\0,]+/, $CGI::values{mv_data_file_field};
     my @file_paths = split /\0/, $CGI::values{mv_data_file_path};
+    my @file_name_from = split /\0/, $CGI::values{mv_data_file_name_from};
     my @file_oldfiles = split /\0/, $CGI::values{mv_data_file_oldfile};
 
 	if($en_col) {
@@ -1900,7 +1958,8 @@ sub update_data {
 	}
 	$function = 'update' unless $function;
 
-	my (%data);
+	my %data;
+	my %sneakdata;
 	for(@fields) {
 		$data{$_} = [];
 	}
@@ -1947,6 +2006,8 @@ sub update_data {
 	}
 #::logDebug("autonumber=$autonumber");
 
+	my $multikey = $base_db->config('COMPOSITE_KEY');
+
  	if(@file_fields) {
 		my $Tag = new Vend::Tags;
 		my $acl_func;
@@ -1972,20 +2033,62 @@ sub update_data {
 		}
 
 		for (my $i = 0; $i < @file_fields; $i++) {
-			unless (length($data{$file_fields[$i]}->[0])) {
+			my $nm = $file_fields[$i];
+
+			next if $nm =~ /__\d+$/;
+			my $dref;
+			my $dmain;
+			my $ntag = '';
+			if($nm =~ m{^(\d+)_} and $CGI::values{$nm}) {
+				$ntag = $1;
+				$ntag .= "_";
+				$sneakdata{$nm}->[0] =  $CGI::values{$nm};
+				for(qw/ mv_data_file_name_to_ mv_data_file_size_to_ /) {
+					my $t = $_ . $nm;
+					my $fld = $CGI::values{$t}
+						or next;
+					$fld = "$ntag$fld";
+					$sneakdata{$fld}->[0] = $CGI::values{$fld};
+				}
+				$dref = $sneakdata{$nm};
+				$dmain = \%sneakdata;
+			}
+			else {
+				$dref = $data{$nm};
+				$dmain = \%data;
+			}
+
+			unless (length($dref->[0])) {
 				# no need for a file update
-				$data{$file_fields[$i]}->[0] = $file_oldfiles[$i];
+				if($file_oldfiles[$i]) {
+					$dref->[0] = $file_oldfiles[$i];
+				}
+				else {
+					$skip_for_now{$nm} = 1;
+				}
 				next;
 			}
 
 			# remove path components
-			$data{$file_fields[$i]}->[0] =~ s:.*/::; 
-			$data{$file_fields[$i]}->[0] =~ s:.*\\::; 
+			$dref->[0] =~ s:.*/::; 
+			$dref->[0] =~ s:.*\\::; 
+
+			if(my $switch = $file_name_from[$i]) {
+				my $new;
+				if($data{$switch} and $new = $data{$switch}->[0]) {
+					my $ext = $dref->[0];
+					if($ext =~ s/.*\.//) {
+						$dref->[0] = join '.', $new, $ext;
+					}
+					else {
+						$dref->[0] = $new;
+					}
+				}
+			}
 
 			if (length ($file_paths[$i])) {
 				# real file upload
-				$outfile = join('/', $file_paths[$i], $data{$file_fields[$i]}->[0]);
-#::logDebug("file upload: field=$file_fields[$i] path=$file_paths[$i] outfile=$outfile");
+				$outfile = join('/', $file_paths[$i], $dref->[0]);
 				my $ok;
 				if (-f $outfile) {
 					eval {
@@ -2007,17 +2110,17 @@ sub update_data {
 				} 
 				my $err;
 				Vend::Interpolate::tag_value_extended(
-										$file_fields[$i],
+										$nm,
 										{
 											test => 'isfile'
 										}
 										)
 					or do {
-						 logError("%s is not a file.", $data{$file_fields[$i]}->[0]);
+						 logError("%s is not a file.", $dref->[0]);
 						 next;
 					};
 				Vend::Interpolate::tag_value_extended(
-						$file_fields[$i],
+						$nm,
 						{
 							outfile => $outfile,
 							umask => $::Scratch->{mv_create_umask} || '022',
@@ -2032,9 +2135,15 @@ sub update_data {
 			}
 			else {
 				# preparing to dump file contents into database column
-				$data{$file_fields[$i]}->[0]
-					= Vend::Interpolate::tag_value_extended ($file_fields[$i],
+				if(my $nfield = $CGI::values{"mv_data_file_name_to_$nm"}) {
+					$dmain->{"$ntag$nfield"}->[0] = $dmain->{$nm}->[0];
+				}
+				$dmain->{$nm}->[0]
+					= Vend::Interpolate::tag_value_extended ($nm,
 						{file_contents => 1});
+				if(my $sfield = $CGI::values{"mv_data_file_size_to_$nm"}) {
+					$dmain->{"$ntag$sfield"}->[0] = length $dmain->{$nm}->[0];
+				}
 			}
 		}
 	}
@@ -2116,8 +2225,8 @@ sub update_data {
 #::logDebug("iteration of update_data:db=$base_db key=$prikey data=" . ::uneval(\%data));
 		@k = (); @v = ();
 		for(keys %data) {
-#::logDebug("iteration of field $_");
 
+			next if $skip_for_now{$_};
 			next unless (length($value = $data{$_}->[$i]) || $CGI::values{mv_update_empty} );
 			push(@k, $_);
 # LEGACY
@@ -2184,7 +2293,7 @@ sub update_data {
 			}
 			while($field = shift @k) {
 				$value = shift @v;
-				next if $field eq $prikey;
+				next if $field eq $prikey and ! $multikey;
 				
 				## DATA IS SET HERE
 				# We are going to set the field unless it is only for
@@ -2218,7 +2327,8 @@ sub update_data {
 
 			for(keys %$qd) {
 #::logDebug("update_data: Getting ready to set_slice");
-				$qret = $qd->{$_}->set_slice($key, $qf->{$_}, $qv->{$_});
+				my $k = $multikey ? undef : $key;
+				$qret = $qd->{$_}->set_slice($k, $qf->{$_}, $qv->{$_});
 				$rows_set[$i] = $qret unless $rows_set[$i];
 			}
 			if($blob) {
@@ -2237,12 +2347,20 @@ sub update_data {
 				if $CGI::values{mv_data_email};
 		}
 	}
+
+	%skip_for_now = ();
+
 	if(my $new = shift(@multis)) {
-#::logDebug("Doing multi for $new");
 		last SETDATA unless length $CGI::values{"${new}_$multiqual"};
 		for(@fields) {
-			my $value = $CGI::values{$_} = $CGI::values{"${new}_$_"};
-			$data{$_} = [ $value ];
+			my $t = $new . "_$_";
+			if($sneakdata{$t}) {
+				$data{$_} = delete $sneakdata{$t};
+			}
+			else {
+				$data{$_} = [ $CGI::values{$_} = $CGI::values{$t} ];
+			 
+			}
 		}
 		redo SETDATA;
 	}
@@ -2251,7 +2369,6 @@ sub update_data {
 	if($CGI::values{mv_data_return_key}) {
 		my @keys = split /\0/, $CGI::values{mv_data_return_key};
 		for(@keys) {
-#::logDebug("return_key, setting $_");
 			$CGI::values{$_} = join("\0", @rows_set);
 		}
 	}
@@ -2289,6 +2406,38 @@ sub update_data {
 			}
 		}
 
+		my @file_death;
+		my %filekill;
+		my %filemove;
+
+		for(my $i = 0; $i < @file_fields; $i++) {
+			push(@file_death, $i), next unless $file_fields[$i] =~ $some;
+			if($file_fields[$i] =~ $this) {
+				my $k = $file_fields[$i];
+				$k =~ s/$this//;
+				$filemove{$file_fields[$i]} = $k;
+			}
+		}
+
+		my $i;
+		while (defined($i = pop @file_death)) {
+			splice @file_fields, $i, 1;
+			splice @file_paths, $i, 1;
+			splice @file_oldfiles, $i, 1;
+		}
+
+		for(@file_fields) {
+			if(my $new = $filemove{$_}) {
+				$_ = $new;
+			}
+
+		}
+
+		while(my ($k,$v) = each %filemove) {
+			$CGI::file{$v} = delete $CGI::file{$k};
+		}
+
+
 		$::Scratch->{mv_data_enable} = delete $::Scratch->{"mv_data_enable__$new"};
 		delete $::Scratch->{mv_data_enable_key};
 
@@ -2298,6 +2447,9 @@ sub update_data {
 		}
 
 		@CGI::values{keys %cgiset} = values %cgiset;
+		$CGI::values{mv_data_file_field} = join "\0", @file_fields;
+		$CGI::values{mv_data_file_path} = join "\0", @file_paths;
+		$CGI::values{mv_data_file_oldfiles} = join "\0", @file_oldfiles;
 #::logDebug("Reloading, function=$CGI::values{mv_data_function}");
 		update_data();
 	}
