@@ -1,8 +1,8 @@
 # Vend::Page - Handle Interchange page routing
 # 
-# $Id: Page.pm,v 2.18 2004/04/02 17:19:20 mheins Exp $
+# $Id: Page.pm,v 2.21 2005/11/08 18:14:45 jon Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -46,7 +46,7 @@ use strict;
 
 use vars qw/$VERSION/;
 
-$VERSION = substr(q$Revision: 2.18 $, 10);
+$VERSION = substr(q$Revision: 2.21 $, 10);
 
 my $wantref = 1;
 
@@ -76,6 +76,7 @@ sub display_special_page {
 	die ::get_locale_message(412, "Missing special page: %s\n", $name)
 		unless defined $page;
 	$page =~ s#\[subject\]#$subject#ig;
+	$Global::Variable->{MV_SUBJECT} = $subject;
 	$Vend::PageInit = 0;
 	interpolate_html($page, 1);
 	::response();
@@ -239,6 +240,7 @@ sub output_cat {
 		$out .= ${$Vend::Output[$_]};
 		undef $Vend::Output[$_];
 	}
+	$out =~ s/^\s+// if $::Pragma->{strip_white};
 	return $out;
 }
 
