@@ -1,7 +1,8 @@
 # Vend::Payment::Linkpoint - Interchange Linkpoint support
 #
-# $Id: Linkpoint.pm,v 1.1 2003/11/22 23:32:01 racke Exp $
+# $Id: Linkpoint.pm,v 1.5 2005/11/08 18:14:46 jon Exp $
 #
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 2002 Stefan Hornburg (Racke) <racke@linuxia.de>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -73,9 +74,9 @@ or
 
 or 
 
-    Variable LINKPOINT_ID YourLinkpointID
+    Variable MV_PAYMENT_ID YourLinkpointID
 
-Required settings are C<host>, C<keyfile> and C<bin>.
+Required settings are C<id> and C<keyfile>.
 
 The active settings are:
 
@@ -83,7 +84,8 @@ The active settings are:
 
 =item host
 
-Your LinkPoint Secure Payment Gateway (LSPG) hostname. Usually secure.linkpt.net (production) or staging.linkpt.net (testing).
+Your LinkPoint Secure Payment Gateway (LSPG) hostname. Usually 
+secure.linkpt.net (production) or staging.linkpt.net (testing).
 
 =item keyfile
 
@@ -91,11 +93,6 @@ File name of the merchant security certificate. This file should contain the
 RSA private key and the certificate, otherwise you get an error like
 "Unable to open/parse client certificate file."
 
-=item bin
-
-File name of the LinkPoint binary. You get an "Unknown error" if you
-specify another executable.
-
 =item id
 
 Store number assigned to your merchant account.
@@ -127,16 +124,21 @@ Make sure you "Require"d the module in interchange.cfg:
 
 =item *
 
-Make sure LPERL is installed and working. You can test to see
+Make sure lpperl (v3.0.012+) is installed and working. You can test to see
 whether your Perl thinks they are:
 
-    perl -MLPERL -e 'print "It works.\n"'
+    perl -Mlpperl -e 'print "It works.\n"'
 
 If it "It works." and returns to the prompt you should be OK (presuming
 they are in working order otherwise).
 
 =item *
 
+Make sure curl is installed and working.  Lpperl uses curl to contact 
+Linkpoint.
+
+=item *
+
 Check the error logs, both catalog and global.
 
 =item *
@@ -168,23 +170,24 @@ packages to Vend::Payment and places things there.
 =head1 AUTHOR
 
 Stefan Hornburg (Racke) <racke@linuxia.de>
+Ron Phipps <rphipps@reliant-solutions.com>
 
 =cut
 						
 BEGIN {
 	eval {
 		package Vend::Payment;
-        	require LPERL or die __PACKAGE__ . " requires LPERL";
+        	require lpperl or die __PACKAGE__ . " requires LPPERL";
 	};
 
 	if ($@) {
-		$msg = __PACKAGE__ . ' requires LPERL';
+		$msg = __PACKAGE__ . ' requires LPPERL';
 		::logGlobal ($msg);
 		die $msg;
 	}
 	
 	::logGlobal("%s payment module initialized", __PACKAGE__)
-		unless $Vend::Quiet;
+		unless $Vend::Quiet or ! $Global::VendRoot;
 }
 
 package Vend::Payment;
@@ -194,12 +197,13 @@ sub linkpoint {
 
 	my $opt;
 	my $keyfile;
+	my $host;
 	
 	if(ref $user) {
 		$opt = $user;
 		$user = $opt->{id} || undef;
 		$keyfile = $opt->{keyfile} || undef;
-		$bin = $opt->{bin} || undef;
+		$host = $opt->{host} || undef;
 	}
 	else {
 		$opt = {};
@@ -215,8 +219,9 @@ sub linkpoint {
 	}
 
 #::logDebug("actual map result: " . ::uneval($actual));
+#::logDebug("opt map result: " . ::uneval($opt));
 
-	# we need to check for customer id, keyfile and binary
+	# we need to check for customer id and keyfile
 	# location, as these are the required parameters
 	
 	if (! $user ) {
@@ -234,32 +239,14 @@ sub linkpoint {
 					   MErrMsg => errmsg('No certificate file'),
 					  );
 	}
-
-	if (! $bin ) {
-		$bin = charge_param('bin')
-			or return (
-					   MStatus => 'failure-hard',
-					   MErrMsg => errmsg('No LinkPoint binary'),
-					  );
+	
+	
+	if (! $host ) {
+		$host = charge_param('host') || 'secure.linkpt.net';
 	}
 
-	unless (-x $bin) {
-		return (
-				MStatus => 'failure-hard',
-				MErrMsg => errmsg('LinkPoint binary not executable'),
-			   );
-	
-	}
-	
-	unless ($opt->{host}) {
-		$opt->{host} = charge_param('host') || 'secure.linkpt.net';
-	}
-	
 	my $precision = $opt->{precision} || 2;
 
-	my $referer   =  $opt->{referer}
-					|| charge_param('referer');
-
 	$actual->{mv_credit_card_exp_month} =~ s/\D//g;
 	$actual->{mv_credit_card_exp_year} =~ s/\D//g;
 	$actual->{mv_credit_card_exp_year} =~ s/\d\d(\d\d)/$1/;
@@ -272,11 +259,11 @@ sub linkpoint {
 	my $transtype = $opt->{transaction} || charge_param('transaction') || 'sale';
 
 	my %type_map = (
-		auth		 	=>	'preauth',
-		authorize		=>	'preauth',
-		mauthcapture 	=>	'sale',
-		mauthonly		=>	'preauth',
-		sale	 		=>	'sale',
+		auth		 	=>	'PREAUTH',
+		authorize		=>	'PREAUTH',
+		mauthcapture 	=>	'SALE',
+		mauthonly		=>	'PREAUTH',
+		sale	 		=>	'SALE',
 	);
 	
 	if (defined $type_map{$transtype}) {
@@ -285,57 +272,72 @@ sub linkpoint {
 
 	$amount = $opt->{total_cost} unless $amount;
 	
+	## If Linkpoint doesn't accept anything but two digits, and
+	## you use something besides Locale foo_FOO frac_digits 2
+	## you might break here
 	if(! $amount) {
 		$amount = Vend::Interpolate::total_cost();
 		$amount = Vend::Util::round_to_frac_digits($amount,$precision);
 	}
-    $amount =~ s/\D//g;
 
-	$amount = int ($amount / 100) . '.' . ($amount % 100);
 	$shipping = Vend::Interpolate::tag_shipping();
 	$subtotal = Vend::Interpolate::subtotal();
 	$salestax = Vend::Interpolate::salestax();
 	$order_id = gen_order_id($opt);
+
+	my $addrnum = $actual->{b_address1};
+	my $bcompany = $Values->{b_company};
+	my $scompany = $Values->{company};
+	
+	$addrnum =~ s/^(\d+).*$//g;
+	$scompany =~ s/\&/ /g;
+	$bcompany =~ s/\&/ /g;
 	
 	my %varmap = ( qw /
-				   baddr1 b_address1
-				   baddr2 b_address2
-				   bcity b_city
-				   bcountry b_country
-				   bname b_name
-				   bstate b_state
-				   bzip b_zip
+				   name b_name
+				   address1 b_address1
+				   address2 b_address2
+				   city b_city
+				   state b_state
+				   zip b_zip
+				   country b_country
 				   email email
 				   phone phone_day
-				   saddr1 address1
-				   saddr2 address2
-				   scity city
-				   scountry country
 				   sname name
+				   saddress1 address1
+				   saddress2 address2
+				   scity city
 				   sstate state
 				   szip zip
+				   scountry country
 				   / );
 
 	my %query =
-		(hostname => $opt->{host},
-		 port => '1139',
-		 storename => $user,
-		 keyfile => $keyfile,		 
+		(host => $host,
+		 port => '1129',
+		 configfile => $user,
+		 keyfile => $keyfile,
+		 result => 'LIVE',
+		 terminaltype => 'UNSPECIFIED',
 		 shipping => $shipping,
 		 chargetotal => $amount,
 		 subtotal => $subtotal,
 		 tax => $salestax,
-		 cardnumber	=> $actual->{mv_credit_card_number},
-		 expmonth => sprintf ("%02d", $actual->{mv_credit_card_exp_month}),
-		 expyear => sprintf ("%02d", $actual->{mv_credit_card_exp_year}),
-		 bcompany => $Values->{company},
-		 scompany => $Values->{company},
-		 debuglevel => $opt->{debuglevel},
+		 vattax => '0.00',
+		 cardnumber => $actual->{mv_credit_card_number},
+		 cardexpmonth => sprintf ("%02d", $actual->{mv_credit_card_exp_month}),
+		 cardexpyear => sprintf ("%02d", $actual->{mv_credit_card_exp_year}),
+		 addrnum => $addrnum,
+		 debbugging => $opt->{debuglevel},
+		 company => $bcompany,
+		 scompany => $scompany, # API is broken for Shipping Company per Linkpoint support
 		);
 
-	for (keys %varmap) {
+    for (keys %varmap) {
         $query{$_} = $actual->{$varmap{$_}};
     }
+    
+    $query{saddress2} = $actual->{address1} . ' ' . $actual->{address2}; # API is broken for Shipping Address Line 1, put line 1 and line 2 in API line 2 per Linkpoint support
 	
     # delete query keys with undefined values
 	for (keys %query) {
@@ -344,28 +346,18 @@ sub linkpoint {
 
 #::logDebug("linkpoint query: " . ::uneval(\%query));
 
-    my $tempfile = "$Vend::Cfg->{ScratchDir}/linkpoint.$order_id";
-	my $lperl = new LPERL($bin, "FILE", $tempfile);
+	my $lperl = new LPPERL();
 
-	my %result;
-
-	if ($transtype eq 'preauth') {
-		%result = $lperl -> CapturePayment (\%query);
-	} elsif ($transtype eq 'sale') {
-		%result = $lperl->ApproveSale(\%query);
-	} else {
-		return (
-				MStatus => 'failure-hard',
-				MErrMsg => errmsg('Unrecognized transaction: %s', $transtype),
-			   );
-	}
+	my %result = $lperl->curl_process(\%query);
 	
 	# Interchange names are on the left, Linkpoint on the right
 	my %result_map = ( qw/
-					   order-id       neworderID
-					   pop.order-id   neworderID
-					   pop.auth-code  code
-					   pop.avs_code   AVSCode
+					   order-id       r_ordernum
+					   pop.order-id   r_ordernum
+					   pop.auth-code  r_code
+					   pop.avs_code   r_avs
+					   pop.status     r_code
+            				   pop.error-message     r_error
 	/
 	);
 
@@ -376,12 +368,16 @@ sub linkpoint {
 			if defined $result{$result_map{$_}};
 	}
 
-	if ($result{statusCode}) {
-		$result{MStatus} = 'success';
-	}
-	else {
+	if ($result{'r_approved'} eq "APPROVED") {
+		$result{'MStatus'} = 'success';
+		$result{'MErrMsg'} = $result{'r_code'};
+	} else {
+		my $msg = errmsg("Charge error: %s Please call in your order or try again.",
+			$result{'r_error'},
+		);
+
 		$result{MStatus} = 'failure';
-	   	$result{MErrMsg} = $result{statusMessage} || 'Unknown error';
+		$result{MErrMsg} = $msg;
 	}
 
 #::logDebug("result given to interchange " . ::uneval(\%result));
