@@ -1,8 +1,8 @@
 # Vend::SQL_Parser - Interchange SQL parser class
 #
-# $Id: SQL_Parser.pm,v 2.9 2004/02/20 01:10:27 jon Exp $
+# $Id: SQL_Parser.pm,v 2.12 2005/11/08 18:14:45 jon Exp $
 #
-# Copyright (C) 2003-2004 Interchange Development Group
+# Copyright (C) 2003-2005 Interchange Development Group
 #
 # Based on HTML::Parser
 # Copyright 1996 Gisle Aas. All rights reserved.
@@ -37,7 +37,8 @@ use strict;
 use Vend::Util;
 use Text::ParseWords;
 use vars qw($VERSION);
-$VERSION = substr(q$Revision: 2.9 $, 10);
+no warnings qw(uninitialized numeric);
+$VERSION = substr(q$Revision: 2.12 $, 10);
 
 sub new {
 	my $class = shift;
@@ -813,7 +814,11 @@ sub new {
 		}
 	}
 	elsif($raw =~ /\s/) {
-		$self->{distinct} = 1 if $raw =~ s/^distinct\s+//i;
+		if ($raw =~ s/^distinct[\s(]+//i) {
+		      $self->{distinct} = 1;
+		      # delete last bracket if exists
+		      $raw =~ s/[\s\)]+$//i;
+		}
 		my $title;
 		$title = $1 if $raw =~ s/\s+as\s+(.*)//;
 		if($title) {
