@@ -1,8 +1,8 @@
 # Vend::Server - Listen for Interchange CGI requests as a background server
 #
-# $Id: Server.pm,v 2.50 2004/03/07 06:59:50 mheins Exp $
+# $Id: Server.pm,v 2.66 2005/11/08 18:14:45 jon Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -26,7 +26,7 @@
 package Vend::Server;
 
 use vars qw($VERSION);
-$VERSION = substr(q$Revision: 2.50 $, 10);
+$VERSION = substr(q$Revision: 2.66 $, 10);
 
 use POSIX qw(setsid strftime);
 use Vend::Util;
@@ -37,6 +37,8 @@ use Socket;
 use Symbol;
 use strict;
 
+no warnings qw(uninitialized);
+
 my $ppidsub = sub { return getppid };
 
 sub new {
@@ -189,6 +191,9 @@ sub map_cgi {
 
 	map_misc_cgi() if $h;
 
+	# Initialize since always used elsewhere, never will be 0
+	$CGI::content_type ||= '';
+
 	my $g = $Global::Selector{$CGI::script_name}
 		or do {
 			my $msg = ::get_locale_message(
@@ -260,11 +265,7 @@ EOF
 	return if $CGI::values{mv_tmp_session};
 
 #::logDebug("Check robot UA=$Global::RobotUA IP=$Global::RobotIP");
-	if ($Global::RobotUA and $CGI::useragent =~ $Global::RobotUA) {
-#::logDebug("It is a robot by UA!");
-		$CGI::values{mv_tmp_session} = 1;
-	}
-	elsif ($Global::RobotIP and $CGI::remote_addr =~ $Global::RobotIP) {
+	if ($Global::RobotIP and $CGI::remote_addr =~ $Global::RobotIP) {
 #::logDebug("It is a robot by IP!");
 		$CGI::values{mv_tmp_session} = 1;
 	}
@@ -278,6 +279,13 @@ EOF
 			$CGI::values{mv_tmp_session} = 1;
 		}
 	}
+	elsif ($Global::NotRobotUA and $CGI::useragent =~ $Global::NotRobotUA) {
+		# do nothing
+	}
+	elsif ($Global::RobotUA and $CGI::useragent =~ $Global::RobotUA) {
+#::logDebug("It is a robot by UA!");
+		$CGI::values{mv_tmp_session} = 1;
+	}
 }
 
 # This is called by parse_multipart
@@ -316,7 +324,7 @@ sub parse_post {
 #::logDebug("found session stuff: $CGI::values{mv_session_id} --> $CGI::values{mv_arg}  --> $CGI::values{mv_pc} ");
 		shift @pairs;
 	}
-	elsif ($#pairs == 1 and $pairs[0] !~ /=/) {	# Must be an isindex
+	elsif (scalar(@pairs) == 1 and $pairs[0] !~ /=/) {	# Must be an isindex
 		$CGI::values{ISINDEX} = $pairs[0];
 		$CGI::values_array{ISINDEX} =  [ split /\+/, $pairs[0] ];
 		@pairs = ();
@@ -375,12 +383,14 @@ sub parse_post {
 
 sub parse_multipart {
 	my $sref = shift;
+
 	my ($boundary) = $CGI::content_type =~ /boundary=\"?([^\";]+)\"?/;
 	$boundary = quotemeta $boundary;
-#::logDebug("got to multipart");
+
 	# Stolen from CGI.pm, thanks Lincoln
 	$boundary = "--$boundary"
 		unless $CGI::useragent =~ /MSIE 3\.0[12];  Mac/i;
+
 	unless ($$sref =~ s/^\s*$boundary\s+//) {
 		die ::errmsg("multipart/form-data sent incorrectly:\n%s\n", $$sref);
 	}
@@ -388,8 +398,6 @@ sub parse_multipart {
 	my @parts;
 	@parts = split /\r?\n$boundary/, $$sref;
 	
-#::logDebug("multipart: " . scalar @parts . " parts");
-
 	DOMULTI: {
 		for (@parts) {	
 		    last if ! $_ || ($_ =~ /^--(\r?\n)?$/);
@@ -500,6 +508,8 @@ sub respond {
 	$$body =~ s/^\s+//
 		if ! $Vend::ResponseMade and $::Pragma->{strip_white};
 
+	$Vend::StatusLine =~ s/\s*$/\r\n/ if $Vend::StatusLine;
+
 	if(! $s and $Vend::StatusLine) {
 		$Vend::StatusLine .= ($Vend::StatusLine =~ /^Content-Type:/im)
 							? '' : "\r\nContent-Type: text/html\r\n";
@@ -554,7 +564,7 @@ sub respond {
 		my $save = select $fh;
 		$| = 1;
 		select $save;
-        $Vend::StatusLine .= "X-Track: " . $Vend::Track->header() . "\r\n"
+        $Vend::StatusLine .= "\r\nX-Track: " . $Vend::Track->header() . "\r\n"
 			if $Vend::Track;
 # END TRACK                            
         $Vend::StatusLine .= "Pragma: no-cache\r\n"
@@ -799,6 +809,8 @@ sub http_soap {
 	if($ref = $Global::Selector{$catname} || $Global::SelectorAlias{$catname}) {
 #::logDebug("found catalog $catname");
 		$$env{SCRIPT_NAME} = $catname;
+	} else {
+		$status = 404;
 	}
 
 	logData("$Global::VendRoot/etc/access_log",
@@ -940,13 +952,17 @@ sub reset_vars {
 	package CGI;
 	reset 'A-Z';
 	reset 'a-z';
+	undef %Vend::Table::DBI::DBI_connect_cache;
+	undef %Vend::Table::DBI::DBI_connect_bad;
+	undef %Vend::Table::DBI::DBI_connect_count;
 	srand();
 #::logDebug("Reset vars");
 }
 
 sub reset_per_fork {
-	%Vend::Table::DBI::DBI_connect_cache = ();
-	%Vend::Table::DBI::DBI_connect_bad = ();
+	undef %Vend::Table::DBI::DBI_connect_cache;
+	undef %Vend::Table::DBI::DBI_connect_bad;
+	undef %Vend::Table::DBI::DBI_connect_count;
 }
 
 sub clean_up_after_fork {
@@ -981,8 +997,8 @@ sub setup_signals {
         $Sig_dec = sub { 1 };
 	}
     else {
-        $Sig_inc = sub { kill "USR1", $Vend::MasterProcess; };
-        $Sig_dec = sub { kill "USR2", $Vend::MasterProcess; };
+        $Sig_inc = sub { kill "USR1", $Vend::MasterProcess || 0; };
+        $Sig_dec = sub { kill "USR2", $Vend::MasterProcess || 0; };
     }
 }
 
@@ -1001,6 +1017,22 @@ sub housekeeping {
 #::logDebug("called housekeeping");
 	return if defined $interval and ($now - $Last_housekeeping < $interval);
 
+	my $do;
+	my $do_before;
+	my $do_after;
+	my $cronjobs;
+
+	if($Global::HouseKeepingCron) {
+		($do, $do_before, $do_after, $cronjobs) = Vend::Cron::housekeeping($now);
+	}
+	else {
+		$do = {
+			restart => 1,
+			reconfig => 1,
+			jobs => 1,
+		};
+	}
+
 #::logDebug("actually doing housekeeping interval=$interval now=$now last=$Last_housekeeping");
 	rand();
 	$Last_housekeeping = $now;
@@ -1022,8 +1054,9 @@ sub housekeeping {
 
 			foreach my $pid (@pids) {
 				kill(0, $pid) and next;
-#::logDebug("Unresponsive server at PID %s", $pid);
+#::logDebug("Non-existent server at PID %s", $pid);
 				push @bad_pids, $pid;
+				delete $Page_pids{$pid};
 			}
 
 			while($count < $Global::StartServers) {
@@ -1033,17 +1066,17 @@ sub housekeeping {
 			}
 			for my $pid (@bad_pids) {
 #::logDebug("Killing excess or unresponsive server at PID %s", $pid);
+				next unless delete $Page_pids{$pid};
 				if(kill 'TERM', $pid) {
 #::logDebug("Server at PID %s terminated OK", $pid);
 					# This is OK
 				}
-				elsif (kill 'TERM', $pid) {
+				elsif (kill 'KILL', $pid) {
 					::logGlobal("page server pid %s required KILL", $pid);
 				}
 				else {
 					::logGlobal("page server pid %s won't die!", $pid);
 				}
-				delete $Page_pids{$pid};
 			}
 		}
 
@@ -1052,10 +1085,25 @@ sub housekeeping {
 		@files = readdir Vend::Server::CHECKRUN;
 		closedir(Vend::Server::CHECKRUN)
 			or die "closedir $Global::RunDir: $!\n";
-		($reconfig) = grep $_ eq 'reconfig', @files;
+		($reconfig) = grep $_ eq 'reconfig', @files
+			if $do->{reconfig};
 		($restart) = grep $_ eq 'restart', @files
 			if $Signal_Restart || $Global::Windows;
-		($jobs) = grep $_ eq 'jobsqueue', @files;
+		($jobs) = grep $_ eq 'jobsqueue', @files
+			if $do->{jobs};
+
+		if($do_before) {
+			for(@$do_before) {
+#::logDebug("run before macro $_");
+				eval {
+					Vend::Dispatch::run_macro($_);
+				};
+				if($@) {
+					::logGlobal("cron before macro '%s' failed: %s", $_, $@);
+				}
+			}
+		}
+
 		if($Global::PIDcheck) {
 			$Num_servers = 0;
 			@pids = grep /^pid\.\d+$/, @files;
@@ -1071,6 +1119,7 @@ sub housekeeping {
 				or die "lock $Global::RunDir/restart: $!\n";
 			while(<Vend::Server::RESTART>) {
 				chomp;
+#::logDebug("restart file reads line '$_'");
 				my ($directive,$value) = split /\s+/, $_, 2;
 				if($value =~ /<<(.*)/) {
 					my $mark = $1;
@@ -1083,6 +1132,7 @@ EOF
 						last;
 					}
 					chomp $value;
+#::logDebug("restart file reads value '$value'");
 				}
 				eval {
 					if($directive =~ /^\s*(sub)?catalog$/i) {
@@ -1095,8 +1145,15 @@ EOF
 					{
 						::remove_catalog($1);
 					}
+					elsif( $directive =~ /^usertag$/i) {
+						Vend::Config::code_from_file($directive, $value, 'nohup');
+					}
+					elsif( $directive =~ /^codedef$/i) {
+						($directive, $value) = split /\s+/, $value, 2;
+						Vend::Config::code_from_file($directive, $value, 'nohup');
+					}
 					else {
-						::change_global_directive($directive, $value);
+						::change_global_directive("$directive $value");
 					}
 				};
 				if($@) {
@@ -1223,6 +1280,32 @@ EOF
 			}
 		}
 
+		if($cronjobs) {
+			for(@$cronjobs) {
+				s/^=//;
+				my (@job) = split /[\s,\0]+/, $_;
+				eval {
+					run_jobs (@job);
+				};
+
+				if($@) {
+					::logGlobal({ level => 'notice' }, $@);
+				}
+			}
+		}
+
+		if($do_after) {
+			for(@$do_after) {
+#::logDebug("would run after macro $_");
+				eval {
+					Vend::Dispatch::run_macro($_);
+				};
+				if($@) {
+					::logGlobal("cron after macro '%s' failed: %s", $_, $@);
+				}
+			}
+		}
+
 		if($respawn) {
 			if($Global::PreFork) {
 				# We need to respawn all the servers to pick up the new config
@@ -1511,6 +1594,7 @@ sub start_page {
 				send_ipc("register page $$");
 
 				my $next;
+				srand();
 				$::Instance = {};
 
 				reset_per_fork();
@@ -1872,6 +1956,10 @@ my $pretty_vector = unpack('b*', $s_vector);
 			}
 			else {
 #::logDebug("we have our SOAP enable, entity is $entity");
+
+				$::Variable = $Vend::Cfg->{Variable};
+				$::Pragma = $Vend::Cfg->{Pragma};
+
 				($Vend::SessionID, $CGI::cookiehost) = split /:/, $env{SESSION_ID};
 #::logDebug("Received ID=$Vend::SessionID, host='$CGI::cookiehost'");
 				$Vend::NoInterpolate = 1
@@ -2182,6 +2270,9 @@ sub server_both {
 	my $only_ipc = $master_ipc;
 	my $checked_soap;
 	my $cycle;
+
+	no warnings; ## We will last out of loop
+
     for (;;) {
 
 	  my $i = 0;
