@@ -1,8 +1,8 @@
 # Vend::Ship - Interchange shipping code
 # 
-# $Id: Ship.pm,v 2.5 2003/09/12 19:45:49 jon Exp $
+# $Id: Ship.pm,v 2.13 2005/06/09 18:43:58 docelic Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -35,6 +35,8 @@ require Exporter;
 use Vend::Util;
 use Vend::Interpolate;
 use Vend::Data;
+use strict;
+no warnings qw(uninitialized numeric);
 
 use constant MAX_SHIP_ITERATIONS => 100;
 use constant MODE  => 0;
@@ -68,8 +70,7 @@ sub do_error {
 	Vend::Tags->error({ name => 'shipping', set => $msg });
 	unless ($Vend::Cfg->{Limit}{no_ship_message}) {
 		$Vend::Session->{ship_message} ||= '';
-		$Vend::Session->{ship_message} .= ' ' if $Vend::Session->{shipping_error};
-		$Vend::Session->{ship_message} .= $msg;
+		$Vend::Session->{ship_message} .= $msg . ($msg =~ / $/ ? '' : ' ');
 	}
 	return undef;
 }
@@ -83,6 +84,8 @@ sub make_three {
 	return $zone;
 }
 
+use vars qw/%Ship_handler/;
+
 %Ship_handler = (
 		TYPE =>
 					sub { 
@@ -629,8 +632,9 @@ sub shipping {
 	}
 	elsif($field eq 'quantity') {
 #::logDebug("quantity selection");
-    	foreach $i (0 .. $#$Vend::Items) {
-			$total = $total + $Vend::Items->[$i]->{$field};
+    	for (@$Vend::Items) {
+			next unless $_->{quantity};
+			$total = $total + $_->{quantity};
     	}
 	}
 	elsif ( index($field, ':') != -1) {
@@ -705,6 +709,7 @@ sub shipping {
 		elsif ($what =~ /^f\s*(.*)/i) {
 			$formula = $o->{formula} || $1;
 			$formula =~ s/\@\@TOTAL\@\\?\@/$total/ig;
+			$formula =~ s/\@\@CRIT\@\\?\@/$total/ig;
 			$formula = interpolate_html($formula)
 				if $formula =~ /__\w+__|\[\w/;
 			$cost = $Vend::Interpolate::ready_safe->reval($formula);
@@ -755,7 +760,7 @@ sub shipping {
 		}
 		elsif ($what =~ s/^s\s*//) {
 			$what =~ s/\s+(.*)//;
-			$extra = $1;
+			my $extra = $1;
 			my $loc = $Vend::Cfg->{Shipping_repository}{$what}
 				or return do_error("Unknown custom shipping type '%s'", $what);
 			for(keys %$loc) {
@@ -835,7 +840,7 @@ sub shipping {
 
 
 	SHIPFORMAT: {
-		$Vend::Session->{ship_message} .= $error_message
+		$Vend::Session->{ship_message} .= $error_message . ($error_message =~ / $/ ? '' : ' ')
 			if defined $error_message;
 		undef $::Carts->{mv_shipping};
 		$Vend::Items = $save;
@@ -844,10 +849,12 @@ sub shipping {
 #::logDebug("ship options: " . uneval($o) );
 		$final /= $Vend::Cfg->{PriceDivide}
 			if $o->{PriceDivide} and $Vend::Cfg->{PriceDivide} != 0;
+		$o->{free} = interpolate_html($o->{free}) if $o->{free} =~ /[_@[]/;
 		unless ($o->{free}) {
 			return '' if $final == 0;
 			$o->{adder} =~ s/\bx\b/$final/g;
 			$o->{adder} =~ s/\@\@TOTAL\@\\?\@/$final/g;
+			$o->{adder} =~ s/\@\@CRIT\@\\?\@/$total/g;
 			$o->{adder} = $Vend::Interpolate::ready_safe->reval($o->{adder});
 			$final += $o->{adder} if $o->{adder};
 			$final = POSIX::ceil($final) if is_yes($o->{round});
@@ -881,7 +888,7 @@ sub shipping {
 
 		$opt->{format} ||= '%M=%D (%F)' if $opt->{output_options};
 		
-		my $label = $opt->{format} || '<OPTION VALUE="%M"%S>%D (%F)';
+		my $label = $opt->{format} || '<option value="%M"%S>%D (%F)';
 		my $sel = $::Values->{mv_shipmode} eq $mode;
 #::logDebug("label start: $label");
 		my %subst = (
@@ -916,10 +923,11 @@ sub shipping {
 	}
 
 	# If we got here, the mode and quantity fit was not found
-	$Vend::Session->{ship_message} .=
-		"No match found for mode '$mode', quantity '$total', "	.
-		($qual ? "qualifier '$qual', " : '')					.
-		"returning 0. ";
+	$Vend::Session->{ship_message} ||= '';
+	my $fmt = "No match found for mode '%s', quantity '%s', ";
+	$fmt .= "qualifier '%s', " if $qual;
+	$fmt .= "returning 0.";
+	$Vend::Session->{ship_message} .= errmsg($fmt, $mode, $total, $qual);
 	return undef;
 }
 
@@ -1038,6 +1046,22 @@ sub tag_ups {
 	my(@fieldnames);
 	my($i,$point,$zone);
 
+	$weight += $opt->{packaging_weight} if $opt->{packaging_weight};
+
+	if($opt->{source_grams}) {
+		$weight *= 0.00220462;
+	}
+	elsif($opt->{source_kg}) {
+		$weight *= 2.20462;
+	}
+	elsif($opt->{source_oz}) {
+		$weight /= 16;
+	}
+
+	if($opt->{oz}) {
+		$weight *= 16;
+	}
+
 #::logDebug("tag_ups: type=$type zip=$zip weight=$weight code=$code opt=" . uneval($opt));
 
 	if(my $modulo = $opt->{aggregate}) {
@@ -1056,7 +1080,6 @@ sub tag_ups {
 
 	$code = 'u' unless $code;
 
-
 	unless (defined $Vend::Database{$type}) {
 		logError("Shipping lookup called, no database table named '%s'", $type);
 		return undef;
@@ -1130,7 +1153,7 @@ sub tag_ups {
 	}
 	elsif (!$zone or $zone eq '-') {
 		$Vend::Session->{ship_message} .=
-			"No $type shipping allowed for geo code $zip.";
+			"No $type shipping allowed for geo code $zip. ";
 #::logDebug("tag_ups empty zone $zone.");
 		return undef;
 	}
@@ -1140,7 +1163,7 @@ sub tag_ups {
 	$cost += tag_data($type,$zone,$eas_zone)  if defined $eas_point;
 	$Vend::Session->{ship_message} .=
 								errmsg(
-									"Zero cost returned for mode %s, geo code %s.",
+									"Zero cost returned for mode %s, geo code %s. ",
 									$type,
 									$zip,
 								)
