@@ -1,8 +1,8 @@
 # Vend::Table::Common - Common access methods for Interchange databases
 #
-# $Id: Common.pm,v 2.34 2003/11/17 23:41:13 edl Exp $
+# $Id: Common.pm,v 2.43 2005/11/08 18:14:47 jon Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -23,13 +23,14 @@
 # Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 # MA  02111-1307  USA.
 
-$VERSION = substr(q$Revision: 2.34 $, 10);
+$VERSION = substr(q$Revision: 2.43 $, 10);
 use strict;
 
 package Vend::Table::Common;
 require Vend::DbSearch;
 require Vend::TextSearch;
 require Vend::CounterFile;
+no warnings qw(uninitialized numeric);
 use Symbol;
 use Vend::Util;
 
@@ -142,11 +143,14 @@ sub lock_table {
 	my $lockhandle;
 	if(not $lockhandle = $s->[$CONFIG]{_lock_handle}) {
 		my $lf = $s->[$CONFIG]{file} . '.lock';
+		unless($lf =~ m{/}) {
+			$lf = ($s->[$CONFIG]{dir} || $Vend::Cfg->{ProductDir}) . "/$lf";
+		}
 		$lockhandle = gensym;
 		$s->[$CONFIG]{_lock_file} = $lf;
 		$s->[$CONFIG]{_lock_handle} = $lockhandle;
 		open $lockhandle, ">> $lf"
-			or die errmsg("Cannot lock table %s: %s", $s->[$CONFIG]{name}, $!);
+			or die errmsg("Cannot lock table %s (%s): %s", $s->[$CONFIG]{name}, $lf, $!);
 	}
 #::logDebug("lock handle=$lockhandle");
 	Vend::Util::lockfile($lockhandle);
@@ -476,6 +480,7 @@ sub clone_set {
 sub stuff_row {
     my ($s, @fields) = @_;
 	my $key = $fields[$s->[$KEY_INDEX]];
+
 #::logDebug("stuff key=$key");
 	$fields[$s->[$KEY_INDEX]] = $key = $s->autonumber()
 		if ! length($key);
@@ -749,7 +754,6 @@ sub query {
 #::logDebug("receieved query. object=" . ::uneval_it($opt));
 
 	if(defined $opt->{values}) {
-		# do nothing
 		@arg = $opt->{values} =~ /['"]/
 				? ( Text::ParseWords::shellwords($opt->{values})  )
 				: (grep /\S/, split /\s+/, $opt->{values});
@@ -771,7 +775,7 @@ sub query {
 	my $return;
 	my $spec;
 	my $stmt;
-	my $update;
+	my $update = '';
 	my %nh;
 	my @na;
 	my @update_fields;
@@ -931,7 +935,8 @@ eval {
 	}
 	else {
 #::logDebug(	" \$Vend::Interpolate::Tmp->{$opt->{arrayref}}");
-		$ref = $Vend::Interpolate::Tmp->{$opt->{arrayref}} = $search->array($spec);
+		$ref = $Vend::Interpolate::Tmp->{$opt->{arrayref} || ''}
+			 = $search->array($spec);
 		$opt->{object} = $search;
 		$opt->{prefix} = 'sql' unless defined $opt->{prefix};
 	}
@@ -950,6 +955,13 @@ eval {
 		$return = $opt->{failure} || undef;
 	}
 
+	if($opt->{search_label}) {
+		$::Instance->{SearchObject}{$opt->{search_label}} = {
+			mv_results => $ref,
+			mv_field_names => \@na,
+		};
+	}
+
 	if ($opt->{row_count}) {
 		my $rc = $ref ? scalar @$ref : 0;
 		return $rc unless $opt->{list};
@@ -1172,6 +1184,10 @@ EndOfExcel
 				my $option = $1;
 				push @o, $1;
 			}
+			elsif (exists $options->{INDEX_OPTIONS}{$f}) {
+
+				push @o, $options->{INDEX_OPTIONS}{$f};
+			}
 			else {
 				push @o, '';
 			}
@@ -1206,6 +1222,11 @@ EndOfExcel
 				if($o[$i] =~ s/c//) {
 					$index .= <<EndOfIndex;
 			map { print { \$fh[$i] } "\$_\\t\$fields[0]\\n" } split /\\s*,\\s*/, \$fields[$fnum];
+EndOfIndex
+				}
+				elsif($o[$i] =~ s/s//) {
+					$index .= <<EndOfIndex;
+			map { print { \$fh[$i] } "\$_\\t\$fields[0]\\n" } split /\\s*;\\s*/, \$fields[$fnum];
 EndOfIndex
 				}
 				else {
@@ -1354,7 +1375,7 @@ EndOfRoutine
 );
 
     eval $format{$format};
-	die errmsg("%s import failed: %s", $options->{name}, $@) if $@;
+	die errmsg("%s import into %s failed: %s", $options->{name}, $options->{table}, $@) if $@;
     if($realfile) {
 		close IN
 			or die errmsg("%s %s: %s\n", errmsg("close"), $infile, $!);
@@ -1571,7 +1592,7 @@ sub log_error {
 		logGlobal($msg);
 	}
 	if($Vend::admin or ! defined($cfg->{LOG_ERROR_SESSION}) or $cfg->{LOG_ERROR_SESSION}) {
-		$Vend::Session->{errors} = {} unless ref($Vend::Session->{errors}) eq 'HASH';
+		$Vend::Session->{errors} = {} unless CORE::ref($Vend::Session->{errors}) eq 'HASH';
 		$Vend::Session->{errors}{$ekey} = $msg;
 	}
 	die $msg if $cfg->{DIE_ERROR};
