@@ -1,8 +1,8 @@
 # Vend::Table::DBI - Access a table stored in an DBI/DBD database
 #
-# $Id: DBI.pm,v 2.58 2004/03/31 15:52:36 racke Exp $
+# $Id: DBI.pm,v 2.68 2005/11/08 18:14:47 jon Exp $
 #
-# Copyright (C) 2002-2004 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -21,9 +21,10 @@
 # MA  02111-1307  USA.
 
 package Vend::Table::DBI;
-$VERSION = substr(q$Revision: 2.58 $, 10);
+$VERSION = substr(q$Revision: 2.68 $, 10);
 
 use strict;
+no warnings qw(uninitialized numeric);
 
 # 0: dummy open object
 # 1: table name
@@ -277,13 +278,14 @@ sub create_sql {
 	$keycol = 0 unless defined $keycol;
 	$config->{KEY_INDEX} ||= $keycol;
 	$config->{KEY} ||= $key;
+	$config->{KEY_DEF} ||= 'char (16) NOT NULL';
 
 	if ( not defined $config->{COLUMN_DEF}->{$key} ) {
 		if($config->{AUTO_SEQUENCE} and $config->{SEQUENCE_KEY}) {
 			$cols[$keycol] =~ s/\s+.*/ $config->{SEQUENCE_KEY}/;
 		}
-		else {
-			$cols[$keycol] =~ s/\s+.*/ char(16) NOT NULL/;
+		elsif(! $config->{COMPOSITE_KEY}) {
+			$cols[$keycol] =~ s/\s+.*/ $config->{KEY_DEF}/;
 		}
 	}
 
@@ -432,13 +434,16 @@ sub create {
 	}
 
 	my @index;
+	my $key_index_found;
 	if(ref $config->{INDEX}) {
 		for my $def (@{$config->{INDEX}}) {
 			my $uniq = '';
 			$uniq = 'UNIQUE' if $def =~ s/^\s*unique\s+//i;
-			$def =~ s/:\w+//g;
+			$def =~ s/:(\w+)//g
+				and $config->{INDEX_OPTIONS}{$def} = $1;
 			my $col = $def;
 			$col =~ s/\W.*//s;
+			$key_index_found = 1 if lc($col) eq lc($key);
 			my $template = $config->{ALTER_INDEX}
 						|| $known_capability{ALTER_INDEX}{default};
 			$template =~ s/\b_TABLE_\b/$tablename/g;
@@ -462,15 +467,17 @@ sub create {
 					);
 			$db->commit() if $config->{Transactions};
 		}
-	} elsif ($config->{AUTO_INDEX_PRIMARY_KEY}) {
+	}
+	elsif ($config->{AUTO_INDEX_PRIMARY_KEY}) {
 		# Oracle automatically creates indexes on primary keys,
 		# so we don't need to do it again
-	} else {
+	}
+	elsif(! $key_index_found) {
 		$db->do("create index ${tablename}_${key} on $tablename ($key)")
 			or ::logError("table %s index failed: %s" , $tablename, $DBI::errstr);
 		$db->commit() if $config->{Transactions};
 	}
-
+ 
 	for(@index) {
 #::logDebug("Running: $_");
 		$db->do($_) 
@@ -1384,7 +1391,7 @@ sub set_row {
     $s->bind_entire_row($cfg->{_Insert_h}, @fields);
 
 	my $rc = $cfg->{_Insert_h}->execute()
-		or die $s->log_error("%s error: $DBI::errstr", 'set_row', $DBI::errstr);
+		or die $s->log_error("%s error on key '%s': $DBI::errstr", 'set_row', $fields[$ki], $DBI::errstr);
 
 	$val	= $cfg->{AUTO_SEQUENCE}
 			?  $s->last_sequence_value($fields[$ki])
@@ -1736,8 +1743,8 @@ sub list_fields {
 	my($db, $name, $config) = @_;
 	my @fld;
 
-	my $q = "select * from $name";
-	$q .= " limit 1" if $config->{HAS_LIMIT};
+	my $q = "SELECT * FROM $name WHERE 2 = 1";
+
 	my $sth = $db->prepare($q)
 		or die ::errmsg("%s prepare on %s: %s", 'list_fields', $name, $DBI::errstr);
 
@@ -1762,7 +1769,7 @@ sub list_fields {
 			}
 		};
 	}
-	my @num = map { exists $config->{NUMERIC}{$_} ? DBI::SQL_INTEGER : undef } @fld;
+	my @num = map { exists $config->{NUMERIC}{$_} ? DBI::SQL_NUMERIC : undef } @fld;
 	$config->{_Numeric_ary} = \@num;
 	if($config->{UPPERCASE}) {
 		@fld = map { lc $_ } @fld;
@@ -2094,6 +2101,14 @@ eval {
 	if ($rc < 1 and CORE::ref($ref) and scalar(@$ref) ) {
 		$rc = scalar(@$ref);
 	}
+
+	if($opt->{search_label}) {
+		$::Instance->{SearchObject}{$opt->{search_label}} = {
+			mv_results => $ref,
+			mv_field_names => \@na,
+		};
+	}
+
 	# force to numeric, to handle DBI "0E0"-type results
 	$::Values->{mv_search_match_count} = $rc + 0;
 	if ($opt->{row_count}) {
@@ -2114,45 +2129,56 @@ eval {
 
 sub auto_config {
 	my $string = shift;
-	my ($dsn, $user, $pass) = Text::ParseWords::shellwords($string);
+	my ($dsn, $user, $pass, $catalog, $schema, $name, $type) = Text::ParseWords::shellwords($string);
 	my $handle = DBI->connect($dsn, $user, $pass)
 		or ::logDebug(::errmsg("DatabaseAuto DSN '%s' does not connect.", $dsn));
-	my $schema;
 	my @tabs;
 	my @out;
-	eval {
-		require DBIx::DBSchema;
-		$schema = new_native DBIx::DBSchema $handle;
-	};
-
 	my $sth;
 	eval {
-		$sth = $handle->table_info()
+		$sth = $handle->table_info($catalog, $schema, $name, $type)
 			or die "Table info not enabled for this driver.\n";
 		while(my $ref = $sth->fetchrow_arrayref) {
-			next unless $ref->[3] eq 'TABLE';
+			next unless $ref->[3] eq 'TABLE' or $type;
 			push @tabs, $ref->[2];
 		}
 	};
 
+	my $re;
+	if(exists $Vend::Cfg->{DatabaseAutoIgnore}) {
+		$re = $Vend::Cfg->{DatabaseAutoIgnore};
+	}
+	elsif(exists $Vend::Config::C->{DatabaseAutoIgnore}) {
+		 $re = $Vend::Config::C->{DatabaseAutoIgnore};
+	}
+#::logDebug("ignore re=$re");
+	$re and $re = qr/$re/;
+
 	my %found;
 	return undef unless @tabs;
 	for my $t (@tabs) {
+		next if $re and $t =~ $re;
 		$found{$t} = 1;
 		push @out, [$t, "$t.txt $dsn"];
 		push @out, [$t, "USER $user"] if $user;
 		push @out, [$t, "PASS $pass"] if $pass;
 	}
 
-	if($schema) {
-		for my $create ($schema->sql($handle)) {
-			$create =~ /^CREATE\s+TABLE\s+(\w+)\s+/
+	my $dbschema;
+	eval {
+		require DBIx::DBSchema;
+		$dbschema = DBIx::DBSchema->new_native($handle);
+	};
+	if ($dbschema) {
+		for my $sql ($dbschema->sql($handle)) {
+			$sql =~ /^CREATE\s+TABLE\s+(\w+)\s+/
 				or next;
 			my $t = $1;
 			next unless $found{$t};
-			push @out, [ $t, "CREATE_SQL $create"];
+			push @out, [ $t, "CREATE_SQL $sql" ];
 		}
 	}
+
 	return @out;
 }
 
