@@ -1,8 +1,8 @@
 # Vend::UserDB - Interchange user database functions
 #
-# $Id: UserDB.pm,v 2.27.2.1 2004/04/19 14:32:59 mheins Exp $
+# $Id: UserDB.pm,v 2.41 2005/11/08 18:14:45 jon Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
 
 package Vend::UserDB;
 
-$VERSION = substr(q$Revision: 2.27.2.1 $, 10);
+$VERSION = substr(q$Revision: 2.41 $, 10);
 
 use vars qw!
 	$VERSION
@@ -30,6 +30,7 @@ use Vend::Data;
 use Vend::Util;
 use Safe;
 use strict;
+no warnings qw(uninitialized numeric);
 
 my $ready = new Safe;
 
@@ -538,7 +539,7 @@ sub set_db {
 	}
 
 	for(@fields) {
-		if(defined $ignore{$_}) {
+		if($ignore{$_}) {
 			$self->{PRESENT}->{$_} = 1;
 			next;
 		}
@@ -598,9 +599,12 @@ sub clear_values {
 }
 
 sub get_values {
-	my($self, @fields) = @_;
+	my($self, $valref, $scratchref) = @_;
 
-	@fields = @{ $self->{DB_FIELDS} } unless @fields;
+	$valref = $::Values unless ref($valref);
+	$scratchref = $::Scratch unless ref($scratchref);
+
+	my @fields = @{ $self->{DB_FIELDS} };
 
 	my $db = $self->{DB}
 		or die errmsg("No user database found.");
@@ -635,6 +639,17 @@ sub get_values {
 				 ? $row->{$self->{LOCATION}->{OUTBOARD_KEY}}
 				 : $self->{USERNAME};
 
+	if(my $ef = $self->{OPTIONS}->{extra_fields}) {
+		my @s = grep /\w/, split /[\s,]+/, $ef;
+		my $field = $self->{LOCATION}{PREFERENCES};
+		my $loc   = $self->{OPTIONS}{extra_selector} || 'default';
+		my $hash = get_option_hash($row->{$field});
+		if($hash and $hash = $hash->{$loc} and ref($hash) eq 'HASH') {
+			for(@s) {
+				$::Values->{$_} = $hash->{$_};
+			}
+		}
+	}
 
 	for(@fields) {
 		if($ignore{$_}) {
@@ -672,13 +687,14 @@ sub get_values {
 }
 
 sub set_values {
-	my($self) = @_;
+	my($self, $valref, $scratchref) = @_;
 
-	my @fields;
+	$valref = $::Values unless ref($valref);
+	$scratchref = $::Scratch unless ref($scratchref);
 
 	my $user = $self->{USERNAME};
 
-	@fields = @{$self->{DB_FIELDS}};
+	my @fields = @{$self->{DB_FIELDS}};
 
 	my $db = $self->{DB};
 
@@ -704,16 +720,35 @@ sub set_values {
 	my @bvals;
 
   eval {
+
+	my @extra;
+
+	if(my $ef = $self->{OPTIONS}->{extra_fields}) {
+		my $row = $db->row_hash($user);
+		my @s = grep /\w/, split /[\s,]+/, $ef;
+		my $field = $self->{LOCATION}{PREFERENCES};
+		my $loc   = $self->{OPTIONS}{extra_selector} || 'default';
+		my $hash = get_option_hash( $row->{$field} ) || {};
+
+		my $subhash = $hash->{$loc} ||= {};
+		for(@s) {
+			$subhash->{$_} = $valref->{$_};
+		}
+
+		push @extra, $field;
+		push @extra, uneval_it($hash);
+	}
+
 	for( @fields ) {
-#::logDebug("set_values saving $_ as $::Values->{$_}\n");
+#::logDebug("set_values saving $_ as $valref->{$_}\n");
 		my $val;
 		if ($scratch{$_}) {
-			$val = $::Scratch->{$_}
-				if defined $::Scratch->{$_};	
+			$val = $scratchref->{$_}
+				if defined $scratchref->{$_};	
 		}
 		else {
-			$val = $::Values->{$_}
-				if defined $::Values->{$_};	
+			$val = $valref->{$_}
+				if defined $valref->{$_};	
 		}
 
 		next if ! defined $val;
@@ -728,7 +763,7 @@ sub set_values {
 		}
 		else {
 			::logDebug( errmsg(
-							"cannot set unknown userdb field $_ to: %s",
+							"cannot set unknown userdb field %s to: %s",
 							$_,
 							$val,
 						)
@@ -736,6 +771,11 @@ sub set_values {
 		}
 	}
 	
+	while(@extra) {
+		push @bfields, shift @extra;
+		push @bvals, shift @extra;
+	}
+
 	if(@bfields) {
 		$db->set_slice($user, \@bfields, \@bvals);
 	}
@@ -1102,7 +1142,7 @@ sub login {
 			@$ary == 1
 				or do {
 					$self->log_either(errmsg(
-						"Denied attempted login with nonexistent (indirect from %s) user name %s",
+						@$ary ? "Denied attempted login with ambiguous (indirect from %s) user name %s" : "Denied attempted login with nonexistent (indirect from %s) user name %s",
 						$foreign,
 						$uname,
 						$self->{USERNAME},
@@ -1226,7 +1266,16 @@ sub login {
 	$Vend::login_table = $Vend::Session->{login_table} = $self->{DB_ID};
 	$Vend::username = $Vend::Session->{username} = $self->{USERNAME};
 	$Vend::Session->{logged_in} = 1;
-	
+
+	if (my $macros = $self->{OPTIONS}{postlogin_action}) {
+		eval {
+			Vend::Dispatch::run_macro $macros;
+		};
+		if ($@) {
+			logError("UserDB postlogin_action execution error: %s\n", $@);
+		}
+	}
+
 	1;
 }
 
@@ -1494,7 +1543,7 @@ sub new_account {
 		if ($foreign) {
 			my $uname = ($self->{PASSED_USERNAME} ||= $self->{USERNAME});
 			$uname = $udb->quote($uname);
-			my $q = "select username from $self->{DB_ID} where $foreign = $uname";
+			my $q = "select $foreign from $self->{DB_ID} where $foreign = $uname";
 			my $ary = $udb->query($q)
 				or do {
 					my $msg = errmsg( "Database access error for query: %s", $q);
@@ -1511,13 +1560,6 @@ sub new_account {
 			die errmsg("Username already exists.") . "\n";
 		}
 
-		my $pass = $udb->set_field(
-						$self->{USERNAME},
-						$self->{LOCATION}{PASSWORD},
-						$pw,
-						);
-		die errmsg("Database access error.") . "\n" unless defined $pass;
-
 		if($foreign) {
 			 $udb->set_field(
 						$self->{USERNAME},
@@ -1527,10 +1569,18 @@ sub new_account {
 				or die errmsg("Database access error.");
 		}
 
+		my $pass = $udb->set_field(
+						$self->{USERNAME},
+						$self->{LOCATION}{PASSWORD},
+						$pw,
+						);
+		die errmsg("Database access error.") . "\n" unless defined $pass;
+
 		if($options{no_login}) {
 			$Vend::Session->{auto_created_user} = $self->{USERNAME};
 		}
 		else {
+			$self->{USERNAME} = $foreign if $foreign;
 			username_cookies($self->{USERNAME}, $pw) 
 				if $Vend::Cfg->{CookieLogin};
 
@@ -1628,8 +1678,39 @@ sub get_cart {
 
 	if($opt->{merge}) {
 		$to = [] unless ref $to;
+		my %used;
+		my %alias;
+		my $max;
+
+		for(@$to) {
+			my $master;
+			next unless $master = $_->{mv_mi};
+			$used{$master} = 1;
+			$max = $master if $master > $max;
+		}
+
+		$max++;
+
+		my $rename;
+		my $alias = 100;
+		for(@$cart) {
+			my $master;
+			next unless $master = $_->{mv_mi};
+			next unless $used{$master};
+
+			if(! $_->{mv_si}) {
+				$alias{$master} = $max++;
+				$_->{mv_mi} = $alias{$master};
+			}
+			else {
+				$_->{mv_mi} = $alias{$master};
+			}
+		}
+
 		push(@$to,@$cart);
-	} else {
+
+	}
+	else {
 		@$to = @$cart;
 	}
 }
