@@ -1,8 +1,8 @@
 # Vend::Util - Interchange utility functions
 #
-# $Id: Util.pm,v 2.71 2004/03/07 03:14:41 mheins Exp $
+# $Id: Util.pm,v 2.87.2.1 2005/12/23 15:47:52 kwalsh Exp $
 # 
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -78,6 +78,7 @@ require Exporter;
 );
 
 use strict;
+no warnings qw(uninitialized numeric);
 use Config;
 use Fcntl;
 use Errno;
@@ -87,7 +88,7 @@ use Safe;
 use Vend::File;
 use subs qw(logError logGlobal);
 use vars qw($VERSION @EXPORT @EXPORT_OK);
-$VERSION = substr(q$Revision: 2.71 $, 10);
+$VERSION = substr(q$Revision: 2.87.2.1 $, 10);
 
 my $Eval_routine;
 my $Eval_routine_file;
@@ -530,7 +531,7 @@ sub random_string {
 
     $r = '';
     for ($i = 0;  $i < $len;  ++$i) {
-	$r .= substr($random_chars, int(rand(length($random_chars))), 1);
+		$r .= substr($random_chars, int(rand(length($random_chars))), 1);
     }
     $r;
 }
@@ -658,6 +659,7 @@ eval {
 # See if Data::Dumper is installed with XSUB
 # If it is, session writes will be about 25-30% faster
 eval {
+		die if $ENV{MINIVEND_NO_DUMPER};
 		require Data::Dumper;
 		import Data::Dumper 'DumperX';
 		$Data::Dumper::Indent = 1;
@@ -707,7 +709,7 @@ sub logData {
 		close(MVLOGDATA) or die "close\n";
     };
     if ($@) {
-		::logError ("Could not %s log file '%s': %s\nto log this data:\n%s",
+		logError ("Could not %s log file '%s': %s\nto log this data:\n%s",
 				$@,
 				$file,
 				$!,
@@ -886,6 +888,20 @@ sub get_option_hash {
 	return \%hash;
 }
 
+sub word2ary {
+	my $val = shift;
+	return $val if ref($val) eq 'ARRAY';
+	my @ary = grep /\w/, split /[\s,\0]+/, $val;
+	return \@ary;
+}
+
+sub ary2word {
+	my $val = shift;
+	return $val if ref($val) ne 'ARRAY';
+	@$val = grep /\w/, @$val;
+	return join " ", @$val;
+}
+
 ## Takes an IC scalar form value (parm=val\nparm2=val) and translates it
 ## to a reference
 
@@ -1041,7 +1057,7 @@ sub readin {
 	$file =~ s#\s+$##;
 	$file =~ s#\.html?$##;
 	if($file =~ m{\.\.} and $file =~ /\.\..*\.\./) {
-		::logError( "Too many .. in file path '%s' for security.", $file );
+		logError( "Too many .. in file path '%s' for security.", $file );
 		$file = find_special_page('violation');
 	}
 	$file =~ s#//+#/#g;
@@ -1178,10 +1194,6 @@ sub vendUrl {
 	my $secure;
 	my @parms;
 
-	if(exists $Vend::Cfg->{AlwaysSecure}{$path}) {
-		$r = $Vend::Cfg->{SecureURL};
-	}
-
 	my %skip = qw/form 1 href 1 reparse 1/;
 
 	for(@scratches) {
@@ -1213,10 +1225,6 @@ sub vendUrl {
 	$ct = ++$Vend::Session->{pageCount}
 		unless $opt->{no_count};
 
-	if($opt->{match_security}) {
-		$opt->{secure} = $CGI::secure;
-	}
-
 	if($opt->{no_session}) {
 		undef $id;
 		undef $ct;
@@ -1228,6 +1236,10 @@ sub vendUrl {
 			and $path = "$cur/$path";
 	}
 
+	if($opt->{match_security}) {
+		$opt->{secure} = $CGI::secure;
+	}
+
 	if($opt->{secure} or exists $Vend::Cfg->{AlwaysSecure}{$path}) {
 		$r = $Vend::Cfg->{SecureURL};
 	}
@@ -1424,7 +1436,7 @@ sub check_security {
 		}
 		if($Vend::Cfg->{UserDB} and $Vend::Cfg->{UserDB}{log_failed}) {
 			my $besthost = $CGI::remote_host || $CGI::remote_addr;
-			::logError("auth error host=%s ip=%s script=%s page=%s",
+			logError("auth error host=%s ip=%s script=%s page=%s",
 							$besthost,
 							$CGI::remote_addr,
 							$CGI::script_name,
@@ -1687,7 +1699,25 @@ sub logGlobal {
 
 sub logError {
     my $msg = shift;
-	return unless defined $Vend::Cfg;
+	return unless $Vend::Cfg;
+
+	my $opt;
+	if(ref $_[0]) {
+		$opt = shift(@_);
+	}
+	else {
+		$opt = {};
+	}
+
+    if(! $opt->{file}) {
+        my $tag = $opt->{tag} || $msg;
+        if(my $dest = $Vend::Cfg->{ErrorDestination}{$tag}) {
+            $opt->{file} = $dest;
+        }
+    }
+
+	$opt->{file} ||= $Vend::Cfg->{ErrorFile};
+
 	if(@_) {
 		$msg = errmsg($msg, @_);
 	}
@@ -1698,11 +1728,11 @@ sub logError {
 
     $msg = format_log_msg($msg) unless $msg =~ s/^\\//;
 
-	$Vend::Errors .= $msg if ($Vend::Cfg->{DisplayErrors} ||
-							  $Global::DisplayErrors);
+	$Vend::Errors .= $msg
+		if $Vend::Cfg->{DisplayErrors} || $Global::DisplayErrors;
 
     eval {
-		open(MVERROR, ">>$Vend::Cfg->{ErrorFile}")
+		open(MVERROR, ">> $opt->{file}")
 											or die "open\n";
 		lockfile(\*MVERROR, 1, 1)		or die "lock\n";
 		seek(MVERROR, 0, 2)				or die "seek\n";
@@ -1715,7 +1745,7 @@ sub logError {
 		logGlobal ({ level => 'info' },
 					"Could not %s error file %s: %s\nto report this error: %s",
 					$@,
-					$Vend::Cfg->{ErrorFile},
+					$opt->{file},
 					$!,
 					$msg,
 				);
@@ -1735,9 +1765,12 @@ my %log_sub_map = (
 # First argument should be log type (see above map).
 # Rest of arguments are same as if calling log routine directly.
 sub logOnce {
-	return if exists $logOnce_cache{"@_"};
+	my $tag = join "", @_;
+	return if exists $logOnce_cache{$tag};
 	my $log_sub = $log_sub_map{ lc(shift) } || $log_sub_map{error};
-	$log_sub->(@_) and ++$logOnce_cache{"@_"};
+	my $status = $log_sub->(@_);
+	$logOnce_cache{$tag} = 1;
+	return $status;
 }
 
 
@@ -1775,19 +1808,27 @@ sub read_cookie {
 sub send_mail {
 	my($to, $subject, $body, $reply, $use_mime, @extra_headers) = @_;
 
-	my @headers;
 	if(ref $to) {
 		my $head = $to;
+
+		for(my $i = $#$head; $i > 0; $i--) {
+			if($head->[$i] =~ /^\s/) {
+				my $new = splice @$head, $i, 1;
+				$head->[$i - 1] .= "\n$new";
+			}
+		}
+
 		$body = $subject;
 		undef $subject;
 		for(@$head) {
-			if( /^To:\s*(.+)/ ) {
+			s/\s+$//;
+			if (/^To:\s*(.+)/si) {
 				$to = $1;
 			}
-			elsif (/Reply-to:\s*(.+)/) {
+			elsif (/^Reply-to:\s*(.+)/si) {
 				$reply = $_;
 			}
-			elsif (/^subj(?:ect)?:\s*(.+)/i) {
+			elsif (/^subj(?:ect)?:\s*(.+)/si) {
 				$subject = $1;
 			}
 			elsif($_) {
@@ -1796,6 +1837,25 @@ sub send_mail {
 		}
 	}
 
+	# If configured, intercept all outgoing email and re-route
+	if (
+		my $intercept = $::Variable->{MV_EMAIL_INTERCEPT}
+		                || $Global::Variable->{MV_EMAIL_INTERCEPT}
+	) {
+		my @info_headers;
+		$to = "To: $to";
+		for ($to, @extra_headers) {
+			next unless my ($header, $value) = /^(To|Cc|Bcc):\s*(.+)/si;
+			logError(
+				"Intercepting outgoing email (%s: %s) and instead sending to '%s'",
+				$header, $value, $intercept
+			);
+			$_ = "$header: $intercept";
+			push @info_headers, "X-Intercepted-$header: $value";
+		}
+		$to =~ s/^To: //;
+		push @extra_headers, @info_headers;
+	}
 
 	my($ok);
 #::logDebug("send_mail: to=$to subj=$subject r=$reply mime=$use_mime\n");
@@ -1852,7 +1912,7 @@ sub send_mail {
 
 	SMTP: {
 		my $mhost = $::Variable->{MV_SMTPHOST} || $Global::Variable->{MV_SMTPHOST};
-		my $helo =  $Global::Variable->{MV_HELO};
+		my $helo =  $Global::Variable->{MV_HELO} || $::Variable->{SERVER_NAME};
 		last SMTP unless $none and $mhost;
 		eval {
 			require Net::SMTP;
@@ -1863,8 +1923,7 @@ sub send_mail {
 #::logDebug("using $using");
 		undef $none;
 
-		my $smtp = new Net::SMTP $mhost;
-		$smtp->hello($helo) if $helo;
+		my $smtp = Net::SMTP->new($mhost, Debug => $Global::Variable->{DEBUG}, Hello => $helo);
 #::logDebug("smtp object $smtp");
 
 		my $from = $::Variable->{MV_MAILFROM}
@@ -1876,6 +1935,9 @@ sub send_mail {
 			next unless /^From:\s*(\S.+)$/mi;
 			$from = $1;
 		}
+		push @extra_headers, "From: $from" unless (grep /^From:\s/i, @extra_headers);
+		push @extra_headers, 'Date: ' . POSIX::strftime('%a, %d %b %Y %H:%M:%S %Z', localtime(time())) unless (grep /^Date:\s/i, @extra_headers);
+
 		my $mime = '';
 		$mime = Vend::Interpolate::mime('header', {}, '') if $use_mime;
 		$smtp->mail($from)
@@ -1948,6 +2010,110 @@ sub send_mail {
 	$ok;
 }
 
+sub codedef_routine {
+	my ($tag, $routine, $modifier) = @_;
+
+	my $area = $Vend::Config::tagCanon{lc $tag}
+		or do {
+			logError("Unknown CodeDef type %s", $tag);
+			return undef;
+		};
+
+	$routine =~ s/-/_/g;
+	my @tries;
+	if ($tag eq 'UserTag') {
+		@tries = ($Vend::Cfg->{UserTag}, $Global::UserTag);
+		}
+	else {
+		@tries = ($Vend::Cfg->{CodeDef}{$area}, $Global::CodeDef->{$area});
+	}
+
+	no strict 'refs';
+
+	my $ref;
+
+	for my $base (@tries) {
+		next unless $base;
+	    $ref = $base->{Routine}{$routine}
+			 and return $ref;
+		$ref = $base->{MapRoutine}{$routine}
+		   and return \&{"$ref"};
+	}
+
+	return undef unless $Global::AccumulateCode;
+#::logDebug("trying code_from file for area=$area routine=$routine");
+	$ref = Vend::Config::code_from_file($area, $routine)
+		or return undef;
+#::logDebug("returning ref=$ref for area=$area routine=$routine");
+	return $ref;
+}
+
+sub codedef_options {
+	my ($tag, $modifier) = @_;
+
+	my @out;
+	my $empty;
+
+	my @keys = keys %{$Vend::Cfg->{CodeDef}};
+	push @keys, keys %{$Global::CodeDef};
+
+	my %gate = ( public => 1 );
+
+	my @mod = grep /\w/, split /[\s\0,]+/, $modifier;
+	for(@mod) {
+		if($_ eq 'all') {
+			$gate{private} = 1;
+		}
+
+		if($_ eq 'empty') {
+			$empty = ['', errmsg('--select--')];
+		}
+
+		if($_ eq 'admin') {
+			$gate{admin} = 1;
+		}
+	}
+
+	for(@keys) {
+		if(lc($tag) eq lc($_)) {
+			$tag = $_;
+			last;
+		}
+	}
+
+	my %seen;
+
+	for my $repos ( $Vend::Cfg->{CodeDef}{$tag}, $Global::CodeDef->{$tag} ) {
+		if(my $desc = $repos->{Description}) {
+			my $vis = $repos->{Visibility} || {};
+			my $help = $repos->{Help} || {};
+			while( my($k, $v) = each %$desc) {
+				next if $seen{$k}++;
+				if(my $perm = $vis->{$k}) {
+					if($perm =~ /^with\s+([\w:]+)/) {
+						my $mod = $1;
+						no strict 'refs';
+						next unless ${$mod . "::VERSION"};
+					}
+					else {
+						next unless $gate{$perm};
+					}
+				}
+				push @out, [$k, $v, $help->{$k}];
+			}
+		}
+	}
+
+	if(@out) {
+		@out = sort { $a->[1] cmp $b->[1] } @out;
+		unshift @out, $empty if $empty;
+	}
+	else {
+		push @out, ['', errmsg('--none--') ];
+	}
+	return \@out;
+}
+
 ### Provide stubs for former Vend::Util functions relocated to Vend::File
 *canonpath = \&Vend::File::canonpath;
 *catdir = \&Vend::File::catdir;
